#!/bin/bash

set -e;

VIROLA_USER_NAME="virola"
VIROLA_USER_HOME_DIRECTORY="/home/$VIROLA_USER_NAME"
VIROLA_SYSTEMD_SERVICE_NAME="$VIROLA_USER_NAME-server"
VIROLA_SYSTEMD_SERVICE_FILE_PATH_NAME="/etc/systemd/system/$VIROLA_SYSTEMD_SERVICE_NAME.service"
VIROLA_START_SH_SCRIPT_NAME="start-virola-server.sh"
VIROLA_RESTART_SH_SCRIPT_NAME="restart-virola-server-service.sh"

is_user_exists() {
	if id -u $VIROLA_USER_NAME >/dev/null 2>&1; then
  		return 0
	else
		return 1
	fi
}

is_home_directory_exists() {
	if [ -d $VIROLA_USER_HOME_DIRECTORY ]; then
  		return 0
	else
		return 1
	fi
}

is_virola_server_systemd_configuration_file_exists() {
	if [ -f "$VIROLA_SYSTEMD_SERVICE_FILE_PATH_NAME" ]; then
  		return 0
	else
		return 1
	fi
}

is_virola_server_systemd_running() {

	systemctl daemon-reload
	VIROLA_SERVER_SYSTEMD_STATUS="$(systemctl is-active $VIROLA_SYSTEMD_SERVICE_NAME.service)"
	
	if [ "${VIROLA_SERVER_SYSTEMD_STATUS}" == "active" ]; then
  		return 0
	else
		return 1
	fi
}

create_virola_user_if_not_exists() {

	if ! is_user_exists ; then
		groupadd -f $VIROLA_USER_NAME
		useradd --gid $VIROLA_USER_NAME --create-home --home-dir $VIROLA_USER_HOME_DIRECTORY $VIROLA_USER_NAME
		chown $VIROLA_USER_NAME: $VIROLA_USER_HOME_DIRECTORY
	fi
}

restart_virola_server_systemd() {

	systemctl daemon-reload

	if is_virola_server_systemd_running ; then
		systemctl restart $VIROLA_SYSTEMD_SERVICE_NAME
	fi
}

stop_virola_server_systemd() {

	systemctl daemon-reload

	if is_virola_server_systemd_running ; then
		systemctl stop $VIROLA_SYSTEMD_SERVICE_NAME	
	fi
}

on_virola_server_postinst() {
	
	if is_virola_server_systemd_running ; then
		exit 0
	fi
	
	if is_virola_server_systemd_configuration_file_exists ; then
		exit 0
	fi
	
	YESNO=""
	
	while true; do
	
		echo "Do you want to configure Virola Server? (Y|n) " >/dev/tty
		if exec </dev/tty; then
    			read YESNO;
		fi
		
    		case $YESNO in
        		"" ) ;&
			[Yy] ) break;;
        		[Nn] ) exit 0;;
        	* ) echo "Please answer Y (yes) or N (no).";;
    		esac
	done
	
	while true; do
	
		echo "We recommend registering Virola Server as Systemd service for its automatic startups & gracefull restarts. Do you want to create Systemd configuration file '$VIROLA_SYSTEMD_SERVICE_FILE_PATH_NAME' for Virola Server? (Y|n) " >/dev/tty
    		if exec </dev/tty; then
    			read YESNO;
		fi
		
    		case $YESNO in
        		"" ) ;&
			[Yy] ) break;;
        		[Nn] ) exit 0;;
        	* ) echo "Please answer Y (yes) or N (no).";;
    		esac
	done

	SYSTEMD_CONFIGURAION_USER_NAME="$VIROLA_USER_NAME"
	SYSTEMD_CONFIGURAION_USER_HOME_DIRECTORY="$VIROLA_USER_HOME_DIRECTORY"

	while true; do
	
		echo "We recommend creating non-root User for Virola Server files & start Virola Server process from its name. Do you want to create User '$VIROLA_USER_NAME' with home directory '$VIROLA_USER_HOME_DIRECTORY'? (Y|n) " >/dev/tty
		if exec </dev/tty; then
    			read YESNO;
		fi
    		
    		case $YESNO in
        		"" ) ;&
			[Yy] )
				create_virola_user_if_not_exists; 
				break;;
        		[Nn] ) 
				SYSTEMD_CONFIGURAION_USER_NAME="root"; 
				SYSTEMD_CONFIGURAION_USER_HOME_DIRECTORY="/home" 
				break;;
        	* ) echo "Please answer Y (yes) or N (no).";;
    		esac
	done

	SYSTEMD_CONFIGURAION_VIROLA_SERVER_LISTEN_IP="0.0.0.0"
	SYSTEMD_CONFIGURAION_VIROLA_SERVER_LISTEN_PORT="7777"

	USER_INPUT_IP=""
	while true; do
	
		echo "Enter IP on which Virola Server will listen when running as Systemd service ($SYSTEMD_CONFIGURAION_VIROLA_SERVER_LISTEN_IP): " >/dev/tty
		if exec </dev/tty; then
    			read USER_INPUT_IP;
		fi

		if [ -z "${USER_INPUT_IP}" ]; then
			break;
    		elif [[ "$USER_INPUT_IP" =~ ^(([1-9]?[0-9]|1[0-9][0-9]|2([0-4][0-9]|5[0-5]))\.){3}([1-9]?[0-9]|1[0-9][0-9]|2([0-4][0-9]|5[0-5]))$ ]]; then
  			SYSTEMD_CONFIGURAION_VIROLA_SERVER_LISTEN_IP=$USER_INPUT_IP
			break;
		else
  			echo "Please enter valid IP."
		fi
	done

	USER_INPUT_PORT=""
	while true; do
	
		echo "Enter PORT on which Virola Server will listen when running as Systemd service ($SYSTEMD_CONFIGURAION_VIROLA_SERVER_LISTEN_PORT): " >/dev/tty
		if exec </dev/tty; then
    			read USER_INPUT_PORT;
		fi

		if [ -z "${USER_INPUT_PORT}" ]; then
			break;
    		elif [[ "$USER_INPUT_PORT" =~ ^([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])$ ]]; then
  			SYSTEMD_CONFIGURAION_VIROLA_SERVER_LISTEN_PORT=$USER_INPUT_PORT
			break;
		else
  			echo "Please enter valid PORT."
		fi
	done

	> $VIROLA_SYSTEMD_SERVICE_FILE_PATH_NAME
	echo "[Unit]
Description=Virola Server
Wants=network.target
After=network.target

[Service]
WorkingDirectory=$SYSTEMD_CONFIGURAION_USER_HOME_DIRECTORY
StandardOutput=null
StandardError=null
User=$SYSTEMD_CONFIGURAION_USER_NAME
ExecStart=$SYSTEMD_CONFIGURAION_USER_HOME_DIRECTORY/$VIROLA_START_SH_SCRIPT_NAME
Restart=on-failure

[Install]
WantedBy=multi-user.target" >> $VIROLA_SYSTEMD_SERVICE_FILE_PATH_NAME

	systemctl daemon-reload

	> $SYSTEMD_CONFIGURAION_USER_HOME_DIRECTORY/$VIROLA_START_SH_SCRIPT_NAME
	echo "#!/bin/bash

set -e

cd $SYSTEMD_CONFIGURAION_USER_HOME_DIRECTORY
/usr/bin/virola-server --listen $SYSTEMD_CONFIGURAION_VIROLA_SERVER_LISTEN_IP:$SYSTEMD_CONFIGURAION_VIROLA_SERVER_LISTEN_PORT --storage-dir ./storage --log-dir ./log" >> $SYSTEMD_CONFIGURAION_USER_HOME_DIRECTORY/$VIROLA_START_SH_SCRIPT_NAME

	chmod +x $SYSTEMD_CONFIGURAION_USER_HOME_DIRECTORY/$VIROLA_START_SH_SCRIPT_NAME

	> $SYSTEMD_CONFIGURAION_USER_HOME_DIRECTORY/$VIROLA_RESTART_SH_SCRIPT_NAME
	echo "#!/bin/bash

set -e

sudo systemctl daemon-reload	
sudo systemctl restart $VIROLA_SYSTEMD_SERVICE_NAME" >> $SYSTEMD_CONFIGURAION_USER_HOME_DIRECTORY/$VIROLA_RESTART_SH_SCRIPT_NAME

	chmod +x $SYSTEMD_CONFIGURAION_USER_HOME_DIRECTORY/$VIROLA_RESTART_SH_SCRIPT_NAME

	while true; do
	
		echo "Do you want to start Virola Server Systemd service now? (Y|n) " >/dev/tty
		if exec </dev/tty; then
    			read YESNO;
		fi

    		case $YESNO in
			"" ) ;&		
        		[Yy] ) 
				systemctl enable $VIROLA_SYSTEMD_SERVICE_NAME
				systemctl start $VIROLA_SYSTEMD_SERVICE_NAME
				echo "Congratulations! You can now connect to your Virola Server via:

	HOST: $SYSTEMD_CONFIGURAION_VIROLA_SERVER_LISTEN_IP
	PORT: $SYSTEMD_CONFIGURAION_VIROLA_SERVER_LISTEN_PORT
	USERNAME: admin
	PASSWORD: admin
"
				break;;
        		[Nn] ) 
				echo "To start Virola Server Systemd service later run following commands:

	sudo systemctl enable $VIROLA_SYSTEMD_SERVICE_NAME
	sudo systemctl start $VIROLA_SYSTEMD_SERVICE_NAME
	sudo systemctl status $VIROLA_SYSTEMD_SERVICE_NAME
	"
				break;;
        	* ) echo "Please answer Y (yes) or N (no).";;
    		esac
	done

	exit 0
}

on_virola_server_preremove() {

	stop_virola_server_systemd

	if ! is_user_exists ; then
		echo "Your Virola Server was installed under root User. For safety reasons remove all Virola Server files manually."
		exit 0
	fi

	YESNO=""

	while true; do
	
		echo "Do you want to completely remove Virola Server Database, logs, Systemd files, User '$VIROLA_USER_NAME' & its home directory? (y|N) " >/dev/tty
		if exec </dev/tty; then
    			read YESNO;
		fi
	
    		case $YESNO in
        		[Yy] ) 

				if is_virola_server_systemd_configuration_file_exists ; then
					rm $VIROLA_SYSTEMD_SERVICE_FILE_PATH_NAME
				fi
				
				if is_user_exists ; then
					userdel --remove --force $VIROLA_USER_NAME
				fi

				break;;
        		"" ) break;;
        	* ) echo "Please answer Y (yes) or N (no).";;
    		esac
	done

	exit 0
}

on_virola_server_postupgrade() {
	restart_virola_server_systemd
	exit 0
}
